#ifndef BALL_H
#define BALL_H
 
#include "mbed.h"
#include "N5110.h"
#include "Gamepad.h"


/**
 @brief Ball Class contains necessary parameters to draw the ball; at the appropriate position, with the given size and selected type.
 @author Chukwuebuka Nwibe
 @date May 2018
 */
class Ball
{
public:
    
    
    /**
     *  @details - constructor
     */
    Ball();
    
    
    /**
     *  @details - destructor
     */
    ~Ball();
    
    
    /** Initialise Ball
     *  @details - Initiases the ball with the appropriate different parameters specified by user selections.
     *  @param x - x coordinate for the centre of the ball starting position.
     *  @param y - y coordinate for the centre of the ball starting position.
     *  @param - radius - specifies size of the ball to be drawn.
     *  @param - colour - specifies if the ball is either transparent or filled, depending on ball type selected by user.
     */
    void init(float x, float y, int radius, bool colour);
    
    
    /** Update
     *  @details - Function takes the 2D Vector "position", updates the ball's current position by adding on the values of 'position'.
     *  @param - position - 2D Vector struct from maze engine. It contains values for the centre of the ball for the next iteration.
     */
    void update(Vector2D position);
    
    
    
    /** Draw
     *  @details - Function draws the ball using values for the centre of the ball, radius and ball type selected.
     *  @param - lcd - N5110 Library used to draw the circle representing the ball.
     */
    void draw(N5110 &lcd);
    
    
    /** Get Ball Position
     *  @details - Function returns a 2D Vector struct containing the ball's current position in form (x,y). 
     * This is used when checking for wall collisions in the checkCollision function within the Maze Engine class.
     * This is also used to show ball's position using printf.
     */
    Vector2D getPosition();
    
    
    /** Get Ball Speed
     *  @details - Function returns a 2D Vector struct containing information about the ball's next position.
     *  @return position
     */
    Vector2D getSpeed();
    
    
    /** Set Position
     *  @details - Function gives the position of the ball for the next game update. It is used in the wall collision and maze finish check functions.
     *  @param - p - 2D Vector struct containing the location of the ball to be set to.
     *  @return velocity
     *
     */
    void setPosition(Vector2D p);
    
    
    /** Set speed
     *  @details - This decides where the ball is to travel next.
     *  @param - v - 2D Vector struct which has the location of the ball for the next iteration.
     */
    void setSpeed(Vector2D v);
    
    
private:
 int _radius;
   
    Vector2D velocity;
    
    float _x;
    
    float _y;
    
    bool _colour;
};

#endif