#ifndef MAZEENGINE_H
#define MAZEENGINE_H
 
#include "mbed.h"
#include "N5110.h"
#include "Gamepad.h"
#include "Ball.h"
#include "Maze.h"
#include "Visuals.h"


/**
 @brief Maze Engine Class runs the game loop, draws the mazes ,checks for wall collisions and checks if mazes have been finished.
 @author Chukwuebuka Nwibe
 @date May 2018
 */
class MazeEngine
{
    
public:
    /**
     *  @details - constructor
     */
    MazeEngine(); 
    
    
    /**
     *  @details - destructor
     */
    ~MazeEngine();
    
    
    
    /** Initiase Game Engine
     *   @details - Initiases the game with the appropriate parameters relative to selected game level.
     *   @param mazeModel - selects the particular maze model to be drawn.
     *   @param x - x coordinate for the centre of the ball starting position.
     *   @param y - y coordinate for the centre of the ball starting position.
     *   @param radius - specifies radius of the ball to be drawn.
     *   @param control - selects the ball to be controlled by joystick.
     *   @param colour - gives the ball type; either transparent or filled. Depending on selected ball type.
     */
    void init(int mazeModel, int x, int y, int radius, bool colour, bool control);
    
    
    /** Read Input
     *  @details - Reads the input from the joystick.
     *  @param pad - Gamepad Library.
     */
    void readInput(Gamepad &pad);
    
    
    /** Get Joystick Input
     *  @details - Obtains joystick input using get_mapped_coord(). This input fills 2D Vector position which has values (x,y). 
     *  @param pad - Gamepad Library.
     *  These are they used as the ball's position in the next update of the loop.
     */
    void getJoystickInput(Gamepad &pad); //read joystick input
    
    
    /** Update
     *  @details - Uses position parameters to update game based on Joystick input. Checks is maze is finished, if so, quit function. Otherwise, the position of the ball is updated in next game loop based on Joystick input.
     *  @param position - 2D Vector struct containing ball position.
     */
    void update(N5110 &lcd);
    
    
    /** Draw
     *  @details - Draws both the maze and the ball.
     *  @param lcd - N5110 Library.
     */
    void draw(N5110 &lcd, Visuals &visual);
    
    
    /** checkCollision
    *  @details - gets ball position using "getPosition()" function from the ball class.
    *  The distance between the centre and the outermost turned on pixel (1) is calculated.
    *  If calculated value is different from radius value, there must be a pixel. Stop movement in that direction.
    *  @param lcd - N5110 Library used to create lcd buffer for maze and ball.
    */
    void checkCollision(N5110 &lcd); 
    
    
    /** Get Maze Array
     *  @details - Array used for wall collision checking is gotten. Scans through each pixel in the LCD and sets value in corresponding array to 1 if the pixel is turned on.
     *  @param lcd - N5110 Library.
     *  @return _finish
     */
     void getMazeArray(N5110 &lcd);
    
    
    
    /** Check Finish
     *  @details - Ball position is gotten using "getPosition()" function within Ball class. Maze is finished when x coordinate of ball centre + radius >83 (the maximum value of the LCD buffer).
     *  @param lcd - N5110 Library.
     */
    bool checkFinish();
    
    
    /** Finish
     *  @param _finish - Boolean variable signifying whether the maze has been finished or not.
     */
    bool _finish;
    
    
private:
    
    //create instance of Maze class
    Maze _maze;
    
    //create instance of ball class
    Ball _ball;
    
    //2D vector struct storing ball position
    Vector2D position;
    
    
    int _mazeModel; //generates maze model based on level
    int _x;
    int _y;
    int _radius;
 
    bool _colour; //Boolean variable to define ball type. If _colour = true, the ball(fill) is transparent, otherwise the ball(fill) is solid.
     
    bool _control;
     
     // 2D Float Array storing locations of turned on pixels for wall collision checks.
    float _mazeArray[84][48];
    
    
};
 
#endif